#/bin/bash
# rar-wrap.sh - bash rar wrapper for xarchive
# Copyright (C) 2005 Lee Bigelow <ligelowbee@yahoo.com> 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# set up exit status variables
E_UNSUPPORTED=65

# Supported file extentions for rar 
EXTS="rar cbr"

# Program to wrap
RAR_PROG="rar"

# setup variables opt and archive.
# the shifting will leave the files passed as
# all the remaining args "$@"
opt="$1"
shift 1
archive="$1"
shift 1

# Command line options for prog functions
# disable comments when opening
OPEN_OPTS="v -c-"
ADD_OPTS="a"
NEW_OPTS="a"
REMOVE_OPTS="d"
EXTRACT_OPTS="x"

# the option switches
case "$opt" in
    -i) # info: output supported extentions for progs that exist
        if [ "$(which $RAR_PROG)" ]; then
            for ext in $EXTS; do
                printf "%s;" $ext
            done
        else
            echo command $RAR_PROG not found > /dev/stderr 
            echo extentions $EXTS ignored > /dev/stderr 
        fi
        printf "\n"
        exit
        ;;

    -o) # open: mangle output of rar cmd for xarchive 
        # format of rar output:
#-------------------------------------
# bookmarks/mozilla_bookmarks.html
#                 11512     5231  45% 28-02-05 16:19 -rw-r--r-- F3F3477F m3b 2.9
#-------------------------------------
        
        NAME=0; SIZE=1; DATE=4; TIME=5; ATTR=6; USR=GRP=LINK=9
        # set a[9] blank for unsupported entries
        a[9]="-"

        # set a body counter. rar outputs a header, a line of dashes, content,
        # then another line of dashes. So the first time we hit a line of
        # dashes we increase the body counter to 1 and process all the
        # following lines until we hit another line of dashes and push the
        # body counter past 1.
        let body=0
        # set an array element counter i. If the counter is 0 (NAME), then the
        # whole dataline (the filename) goes into the array element.
        let i=0
        $RAR_PROG $OPEN_OPTS "$archive" | while read dataline; do
            if [ "${dataline:0:4}" == "----" ]; then 
                let body++
            elif [ $body -eq 1 ]; then
                if [ $i -eq $NAME ]; then
                    a[NAME]=$dataline
                    let i++
                else
                    for element in $dataline; do
                        a[i]=$element 
                        let i++
                    done
                    echo "${a[NAME]};${a[SIZE]};${a[ATTR]};${a[USR]};${a[GRP]};${a[DATE]};${a[TIME]};${a[LINK]}" 
                    let i=NAME 
                fi
            fi
        done
        exit
        ;;

    -a) # add:  to archive passed files
        # we only want to add the file's basename, not
        # the full path so...
        while [ "$1" ]; do
            cd "$(dirname "$1")"
            $RAR_PROG $ADD_OPTS "$archive" "$(basename "$1")"
            wrapper_status=$?
            shift 1
        done
        exit $wrapper_status
        ;;

    -n) # new: create new archive with passed files 
        # create will only be passed the first file, the
        # rest will be "added" to the new archive
        cd "$(dirname "$1")"
        $RAR_PROG $NEW_OPTS "$archive" "$(basename "$1")"
        exit
        ;;

    -r) # remove: from archive passed files 
        $RAR_PROG $REMOVE_OPTS "$archive" "$@"
        exit
        ;;

    -e) # extract: from archive passed files 
        # xarchive will put is the right extract dir
        # so we just have to extract.
        $RAR_PROG $EXTRACT_OPTS "$archive" "$@"
        exit
        ;;

    -v) # view: from archive passed files 
        exit E_UNSUPPORTED
        ;;

     *) echo "error, option $opt not supported"
        echo "use one of these:" 
        echo "-i                #info" 
        echo "-o archive        #open" 
        echo "-a archive files  #add" 
        echo "-n archive file   #new" 
        echo "-r archive files  #remove" 
        echo "-e archive files  #extract" 
        echo "-v archive file   #view"  
        exit
esac
